/*
 * Update Daemon for Linux (and possibly other Unixen).
 * Copyright (c) 1996, 1997 Torsten Poulin.
 * Copyright (C) February 1998 Sergei Viznyuk <sergei@phystech.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include <stdlib.h>
#include <signal.h>
#include <syslog.h>

#define	PROGRAM_NAME		"updated"
#define PROGRAM_VERSION		"1.2"
#define DEFAULT_SYNC_DURATION	60

#ifdef NO_BDFLUSH
int	(*update)() = &sync;
#else
#include <sys/kdaemon.h>
int	update_bdflush();
int	(*update)() = &update_bdflush;
/*****************************************************************************/
int update_bdflush()
{
  if ( bdflush(1,0) == -1 )
    {
      openlog(PROGRAM_NAME,LOG_PID,LOG_LOCAL0);
      syslog(LOG_ERR,"bdflush: %m\n");
      syslog(LOG_ERR,"switching to sync(2) instead of bdflush(2)");
      closelog();
      update = &sync;
    }
  return 0;
}
#endif
/*****************************************************************************/
void sigHandler(sig)
int sig;
{
  openlog(PROGRAM_NAME,LOG_PID,LOG_LOCAL0);
  syslog(LOG_ERR,"terminating on signal %d\n",sig);
  sync();
  exit(sig);
}
/*****************************************************************************/
void signalSetup()
{
  int i;
  struct sigaction action;
  sigaction(SIGHUP,NULL,&action);
  action.sa_handler= &sigHandler;
  action.sa_flags = 0;
  for (i=1;i<32;i++) sigaction(i,&action,NULL);
}
/*****************************************************************************/
int main(argn,argc,argv)
int argn;
char *argc[],*argv[];
{
  int s			=	1;
  int k			=	1;
  int i			=	1;
  int sync_duration	=	DEFAULT_SYNC_DURATION;

  if ( getuid() )
    {
      sync();
      fprintf(stderr,"****  %s: not a superuser\n",argc[0]);
      exit(1);
    }

  while ( argc[i] )
    if ( argc[i][0]=='-' )
prgs: switch ( argc[i][s] )
	{
	  case 0:
	    i++;
	    s=1;
	    break;
	  case 'S':
	    s++;
	    update = &sync;
	    goto prgs;
	  case 's':
	    i++;
	    if ( argc[i] )
	      sync_duration=atol(argc[i++]);
	    else
	      goto usage;
	    s=1;
	    if ( sync_duration > 0 ) break;
          default:
usage:	    fprintf(stderr,"\
"PROGRAM_NAME" v."PROGRAM_VERSION"\n\
Copyright (C) 1996, 1997 Torsten Poulin <torsten@diku.dk>\n\
Copyright (C) February 1998 Sergei Viznyuk <sv@phystech.com>\n\
Usage: updated [-S] [-s secs]\n");
	    exit(1);
	}
    else
      argc[k++]=argc[i++];
    
  signalSetup();
  if ( fork() ) exit(0);
  setsid();
  chdir("/");
  for (s=0;s<OPEN_MAX;s++) close(s);
  while ( update() == 0 ) sleep(sync_duration);
  exit(0);
}
